#include "run.h"
#include <windows.h>
#include "consolehelper.h"
#include "textes.h"
#include "windowsUtils.h"

#define EXPLOSION ('X')
#define BALL ('o')
#define BALL_ATTRIBUTES (SKY | RED)

const DWORD keyLeft=VK_LEFT;
const DWORD keyRight=VK_RIGHT;
const DWORD keyUp=VK_UP;
const DWORD keyDown=VK_DOWN;
const DWORD keyShoot=VK_CONTROL;
const DWORD keyTime=VK_TAB;
const DWORD keySave=VK_F5;
const DWORD keyLoad=VK_F6;
const DWORD mouseShoot = FROM_LEFT_1ST_BUTTON_PRESSED;
const DWORD mouseTime = RIGHTMOST_BUTTON_PRESSED;

const int SKY = LIGHTBLUE << 4;

struct SHOOT_INFO {
  int x,y;
  int dir;
};
struct OBJECT_INFO {
  int type; //-2 = nichts, -1 = Erde, 0..15 = Haus (farbe), 16 = explosion, 17 .. 32 = Block
  int life;
};  
struct DESTROYED_OBJECT_INFO {
  int x,y;
  int type;
};

int levelWidth = 100;

#define MAXSHOOTS 80
SHOOT_INFO shoots [MAXSHOOTS];
int shootCount = 0;

int points;

void addShoot (int x,int y, int dir){
  if (shootCount < MAXSHOOTS) {
    shoots[shootCount].x = x;
    shoots[shootCount].y = y;
    shoots[shootCount].dir = dir;
    shootCount++;
  }  
}  

#define MAXDESTROY (80*20)
DESTROYED_OBJECT_INFO destroyed [MAXDESTROY];
int destroyedCount = 0; 

void addDestroyed (int x,int y, OBJECT_INFO * objects, CHAR_INFO * screen){
  if (objects[x+levelWidth*y].type<0) return;
  soundbeep(120,10,false,false,1);
  soundbeep(110,20,false,false,1);
  soundbeep(100,30,false,false,1);
  destroyed[destroyedCount].x=x;
  destroyed[destroyedCount].y=y;
  destroyed[destroyedCount].type=objects[x+levelWidth*y].type;
  if (destroyed[destroyedCount].type<16) points-=20;
  else points+=2;
  objects[x+levelWidth*y].type = 16;
  screen[x+levelWidth*y].Char.AsciiChar=EXPLOSION;
  screen[x+levelWidth*y].Attributes=SKY | RED;
  destroyedCount++;
}  
void deleteDestroyed (int i){
  destroyedCount--;
  CopyMemory(&destroyed[i],&destroyed[i+1],sizeof(destroyed[i])*(destroyedCount-i));
}  
void calcDestroyed (OBJECT_INFO * levelObjects, CHAR_INFO * levelScreen){
  int tempCount=destroyedCount;
  for (int i=destroyedCount-1;i>=0;i--){
    DESTROYED_OBJECT_INFO * destroy = &destroyed[i];
    int pos = destroy->x + levelWidth*destroy->y;
    levelScreen[pos].Char.AsciiChar=' ';
    levelObjects[pos].type = -2;
    if ((destroy->x > 0) && (levelObjects[pos-1].type == destroy->type)) 
      addDestroyed(destroy->x-1,destroy->y,levelObjects,levelScreen);
    if ((destroy->x < levelWidth-1) && (levelObjects[pos+1].type == destroy->type)) 
      addDestroyed(destroy->x+1,destroy->y,levelObjects,levelScreen);
    if ((destroy->y > 0) && (levelObjects[pos-levelWidth].type == destroy->type)) 
      addDestroyed(destroy->x,destroy->y-1,levelObjects,levelScreen);
    if (destroy->y < 21) {
      if (levelObjects[pos+levelWidth].type == destroy->type)
        addDestroyed(destroy->x,destroy->y+1,levelObjects,levelScreen);
    } else {
      levelScreen[pos].Char.AsciiChar='\xDC';
      levelScreen[pos].Attributes=SKY | BROWN;
      levelObjects[pos].type = -1;
    }    
    deleteDestroyed (i);
  }  
  if ((tempCount>0) && (destroyedCount==0)) deleteSounds(1);
}  


void LoadPlainLevel(char * fileName, CHAR_INFO * screenBuffer, OBJECT_INFO * objectBuffer, int * shipX, int * shipY){
  destroyedCount=0;
  shootCount=0;
  
  HANDLE file = CreateFile(fileName,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN, NULL);
  char * buffer = new char [levelWidth+2];
  char * bufpos;
  DWORD readBytes;
  int x=0;
  int y=0;
  for (int i = 0; i<22; ){
    ReadFile(file,&buffer[0],levelWidth+2,&readBytes,NULL);
    bufpos = &buffer[0];
    for (int j = 0;j < readBytes;j++){
      if (*bufpos == '\xD') {
          x=0;
          y++;
          i++;
      } else if ((x<levelWidth) && (y<22))
        if ((*bufpos >= 'a') && (*bufpos <= 'p')) {
          int houseNumber = int (*bufpos - 'a');
          screenBuffer [x+levelWidth*y].Char.AsciiChar = '\xDB';
          screenBuffer [x+levelWidth*y].Attributes = houseNumber << 4 | houseNumber;
          objectBuffer [x+levelWidth*y].type = houseNumber;
          objectBuffer [x+levelWidth*y].life = 15;
          x++;
        } else if ((*bufpos >= 'A') && (*bufpos <= 'P')) {
          int blockNumber = int (*bufpos - 'A');
          screenBuffer [x+levelWidth*y].Char.AsciiChar = '\xDB';
          screenBuffer [x+levelWidth*y].Attributes = blockNumber << 4 | blockNumber;
          objectBuffer [x+levelWidth*y].type = blockNumber+17;
          objectBuffer [x+levelWidth*y].life = 5;
          x++;
        } else if ((*bufpos == '*')) {
          screenBuffer [x+levelWidth*y]=screenBuffer [x+levelWidth*y-1];
          objectBuffer [x+levelWidth*y] = objectBuffer [x+levelWidth*y-1];
          x++;
        } else  if (*bufpos == '_') {
          screenBuffer [x+levelWidth*y].Char.AsciiChar = '\xDC';
          screenBuffer [x+levelWidth*y].Attributes = SKY | BROWN;
          objectBuffer [x+levelWidth*y].type = -1;
          x++;
        } else if (*bufpos=='S'){
          *shipX=x;
          *shipY=y-1;
          screenBuffer [x+levelWidth*y].Char.AsciiChar = ' ';
          screenBuffer [x+levelWidth*y].Attributes = SKY;
          objectBuffer [x+levelWidth*y].type = -2;
          x++;
        } else {if ((*bufpos == '.')||(*bufpos == ' ')){  
          screenBuffer [x+levelWidth*y].Char.AsciiChar = ' ';
          screenBuffer [x+levelWidth*y].Attributes = SKY | BROWN;
          objectBuffer [x+levelWidth*y].type = -2;
          x++;
        } }
      bufpos++;
    }    
    if (readBytes<levelWidth+2) break;
  }  
  delete [] buffer;
  CloseHandle(file);
}  
typedef const char ** ppcc ;
void LoadLevel(char * fileName, CHAR_INFO * &screenBuffer, OBJECT_INFO * &levelBuffer, int * shipX, int * shipY, char * nextLevel, bool & enableBall){
  //quick and dirty
  DWORD read;

  SetLastError(0);
  HANDLE file = CreateFile(fileName,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL|FILE_FLAG_SEQUENTIAL_SCAN,NULL);

  ReadFile(file,&levelWidth, 4,&read,NULL);
  screenBuffer = new CHAR_INFO [levelWidth * 22];
  levelBuffer = new OBJECT_INFO [levelWidth * 22];

  DWORD temp=0;
  SetLastError(0);
  ReadFile(file,&temp, 4,&read,NULL);
  
  char * buffer=new char [temp+1];
  ReadFile(file,&buffer[0], temp,&read,NULL);
  buffer[temp]='\x0';
  LoadPlainLevel(buffer,screenBuffer,levelBuffer,shipX,shipY);
  delete [] buffer;

  ReadFile(file,nextLevel, 16,&read,NULL);

  DWORD lines=0;
  ReadFile(file,&lines,4,&read,NULL);

  ReadFile(file,&temp,4,&read,NULL);
  enableBall=(temp==1);
  
  ReadFile(file,&temp,4,&read,NULL);
  ReadFile(file,&temp,4,&read,NULL);
  ReadFile(file,&temp,4,&read,NULL);

  char * plines = new char[lines*76];
  char ** linesb = new char*[lines];
  ZeroMemory(&plines[0],sizeof(plines[0])*(lines*76));
  ZeroMemory(&linesb[0],sizeof(linesb[0])*lines);
  for (int i=0; i<lines; i++){
    linesb[i]=&plines[i*76];
  }  
  char b;
  int x=0;
  int y=0;
  ReadFile(file,&b,1,&read,NULL);
  while (read>0) {
    if (b=='\xD') {
      x=76*y;
      y++;
      b='\xA';
    }  
    if (b!='\xA') {
      plines[x]=b;
      x++;
    }  
    ReadFile(file,&b,1,&read,NULL);
  } 
  runScrollText(76,lines,ppcc (linesb),0); 
 
  delete [] plines;
  delete [] linesb;

  CloseHandle(file);
//  
}  
bool testShipCollision(OBJECT_INFO* objects, int shipX,int shipY,bool enableBall,bool leftFly){
  const char ** ship = getShip(enableBall,leftFly);
  for (int x=0; x < shipWidth; x++)
    for (int y=0; y < shipHeight; y++)
      if (ship[y][x]!=' ')
        if (objects[x+shipX+levelWidth*(y+shipY)].type>=0) return true;
  return false;
}  
void destroyShip(int realShipX,int realShipY,CHAR_INFO * backBuffer, COORD & top, COORD & right, SMALL_RECT & screen){
  points-=1000;
  for (int i=0;i<50;i++){
    soundbeep(60+i,10,false,false,1);
    soundbeep(50+i,20,false,false,1);
    soundbeep(40+i,30,false,false,1);
  }  
  backBuffer[realShipX+80*(realShipY+1)].Char.AsciiChar='X';
  backBuffer[realShipX+80*(realShipY+1)].Attributes=SKY | RED;
  WriteConsoleOutputA(StdOutput(),&backBuffer[0],right,top,&screen);
  sync(100);
  
  for (int size=2;size<80;size+=2){
    int h=0;
    for (int x=-size;x<=size; x++){
      int nx=realShipX+x;
      if ((nx<0) || (nx>=80)) {
        if (x<0) h++;
        else h--;
        continue;
      }  
      for (int y = -h;y<=h; y++){
        int ny=realShipY+y;
        if ((ny<0)||(ny>=25)) continue;
      
        backBuffer[nx+80*ny].Char.AsciiChar='X';
        backBuffer[nx+80*ny].Attributes=SKY | RED;
      }  
      if (x<0) h++;
      else h--;
    }  
    WriteConsoleOutputA(StdOutput(),&backBuffer[0],right,top,&screen);
    sync(100);
  }  
  for (int x=0; x<80;x++) {
    for (int y=0; y<25;y++){
      backBuffer[x+80*y].Char.AsciiChar='X';
      backBuffer[x+80*y].Attributes=SKY | RED;
    }
  }    
  WriteConsoleOutputA(StdOutput(),&backBuffer[0],right,top,&screen);
  sync(500);             
  deleteSounds(1);  
}  
void playShipStart(){
  for (int i=0;i<125;i++)
    soundbeep(1000+3*i,2,false,false,0);
}
void playShoot(){
  soundbeep(2500,70,false,false,0);
  soundbeep(2300,30,false,false,0);
}  
bool ballKilled;
int scrollPos;
bool moveBallLeft(int &x, int y, int & dir, CHAR_INFO * screen, OBJECT_INFO * objects){
  x--;
  if (x<0) ballKilled=true;
  if (objects[x+levelWidth*y].type>0){
    dir=-dir;
    addDestroyed(x,y,objects,screen);
    x++;
    return false;
  } else return true;
}  
bool moveBallRight(int &x, int y, int & dir, CHAR_INFO * screen, OBJECT_INFO * objects){
  x++;
  if ((x-scrollPos>80) ||(objects[x+levelWidth*y].type>0)){
    dir=-dir;
    if (objects[x+levelWidth*y].type>0)
      addDestroyed(x,y,objects,screen);
    x--;
    return false;
  } else return true;
}  
bool moveBallUp(int x, int &y, int & dir, CHAR_INFO * screen, OBJECT_INFO * objects){
  y--;
  if ((y<0) ||(objects[x+levelWidth*y].type>0)){
    dir=-dir;
    if (y>=0) 
      addDestroyed(x,y,objects,screen);
    y++;
    return false;
  } else return true;
}  
bool moveBallDown(int x, int &y, int & dir, CHAR_INFO * screen, OBJECT_INFO * objects){
  y++;
  if ((y>21) ||(objects[x+levelWidth*y].type>0)){
    dir=-dir;
    if (y<=21) 
      addDestroyed(x,y,objects,screen);
    y--;
    return false;
  } else return true;
}  
struct SCOORD{
   int X;
   int Y;
};
MOUSE_EVENT_RECORD grMouse;
bool newMouse;
void gameRunMouseCallback(MOUSE_EVENT_RECORD*  event){
  grMouse = *event;
  newMouse = true;
}  

void gameRun(){
  char * cBackupBuffer = new char [80*25];
  char * aBackupBuffer = new char [80*25];
  gettext(0,0,79,24,&cBackupBuffer[0],&aBackupBuffer[0]);

  CHAR_INFO * topLines = new CHAR_INFO [80 * 2];
  for (int i=0;i<80;i++){
    topLines[i].Char.AsciiChar=' ';
    topLines[i].Attributes=BLACK << 4 | GREEN; 
    topLines[i+80].Char.AsciiChar='\xDF';
    topLines[i+80].Attributes=SKY | BLACK; 
  }  
  
  CHAR_INFO * levelScreen = NULL;//= new CHAR_INFO [levelWidth * 22];
  OBJECT_INFO * levelObjects = NULL;//= new OBJECT_INFO [levelWidth * 22];

  int lifes=5;
  int energy=100;
  points=0;
  DWORD lastShoot = 0;
  int shipX=0;int shipY=0;
  SCOORD ball, ballDir;
  bool enableBall=true;
  bool leftFly=false;
  ballKilled=false;
  char globLevelFileName [16] = "lev1.lev.info\x0";
  char globNextLevelFileName [18] ="123456789abcde\x0\x0";
  LoadLevel(globLevelFileName,levelScreen,levelObjects,&shipX,&shipY,&globNextLevelFileName[0],enableBall);
  puttextInBuffer(32,0, &topLines[0],16,&globLevelFileName[0],GREEN);
//============

//============
  ball.X=shipX+shipWidth+1;
  ball.Y=shipY+1;
  ballDir.X=2;
  ballDir.Y=0;
 
  CHAR_INFO backBuffer [80*25];
  
  COORD topLeft; topLeft.X = 0; topLeft.Y = 0;
  COORD rightBottom; rightBottom.X = 80; rightBottom.Y = 25;
  SMALL_RECT screen; screen.Left = topLeft.X; screen.Top = topLeft.Y; screen.Right = rightBottom.X-1; screen.Bottom = rightBottom.Y-1 ;

  ZeroMemory(&backBuffer[0],sizeof(backBuffer));
  puttextInBuffer(15,11,&backBuffer[0],51,"Dr\x81""cke die Leertaste um mit der Tastatur zu spielen,  ",BLACK << 4 | GREEN);
  puttextInBuffer(11,12,&backBuffer[0],59,"oder klicke mit der rechten Maustaste um mit ihr zu spielen  ",BLACK << 4 | GREEN);
  WriteConsoleOutputA(StdOutput(),&backBuffer[0],rightBottom,topLeft,&screen);
  bool mouseEnabled=false;
  setMouseCallback(PMouseCallback(gameRunMouseCallback));
  newMouse = false;  
  while (1) {
    if (keypressed(VK_SPACE)) {
      mouseEnabled = false;
      break;
    }  
    if (((grMouse.dwButtonState & RIGHTMOST_BUTTON_PRESSED) == RIGHTMOST_BUTTON_PRESSED) && newMouse){
      mouseEnabled= true ;
      newMouse = false;
      setMousePos(shipX-scrollPos+shipWidth/2,shipY+2+1);
      break;
    }  
  }  

  playShipStart();
  //MessageBoxInt("",GetLastError());
//  drawScrollText(10,10,10+shipWidth-1,10+shipHeight-1,0,3,ship,CYAN,CYAN);
  scrollPos=0;
  int quadroFrame=0;
  int timeTravel=0;
  COORD oldMousePos;
  oldMousePos=grMouse.dwMousePosition;
  while (!keypressed(VK_ESCAPE)) {
    if (mouseEnabled) mouseEnabled=!keypressed(VK_SPACE);
    else {
      mouseEnabled=(((grMouse.dwButtonState & keyShoot) == keyShoot) || ((grMouse.dwButtonState & RIGHTMOST_BUTTON_PRESSED) == RIGHTMOST_BUTTON_PRESSED))&& newMouse;
      newMouse=false;
      if (mouseEnabled) {
        setMousePos(shipX-scrollPos+shipWidth/2,shipY+2+1);
        oldMousePos=grMouse.dwMousePosition;
      }  
    }  
    DWORD ticks = GetTickCount();
    //------DRAW--------
    ZeroMemory(&backBuffer[0],sizeof(backBuffer));
    CopyMemory (&backBuffer[0], &topLines[0], sizeof(topLines[0]) * (80*2));
    for (int y=0;y<22;y++)
      CopyMemory (&backBuffer[160+y*80],&levelScreen[y*levelWidth+scrollPos],sizeof(levelScreen[0])*80);
    
    if (timeTravel>0) {
      puttextInBuffer(32,11,&backBuffer[0],15,"\xC9\xCD\xCDZeitreise\xCD\xCD\xBB",BLACK << 4 | GREEN);
      puttextInBuffer(32,12,&backBuffer[0],15,"\xBA             \xBA",BLACK << 4 | GREEN);
      putIntInBuffer(39,12,&backBuffer[0],timeTravel,BLACK << 4 | GREEN);
      puttextInBuffer(32,13,&backBuffer[0],15,"\xC8\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xBC",BLACK << 4 | GREEN);
    } else {
      int ballpos=ball.X-scrollPos+80*(ball.Y+2);
      if ((enableBall)&&(ball.X>scrollPos)&&(ball.X<levelWidth)&&(ballpos>0)&&(ballpos<80*25)) {
        backBuffer[ballpos].Char.AsciiChar=BALL;
        backBuffer[ballpos].Attributes=BALL_ATTRIBUTES;
      }  
      CopyShipToBuffer(shipX-scrollPos,shipY+2,&backBuffer[0],enableBall,leftFly);
    }  
    puttextInBuffer(1,24,&backBuffer[0],22,"m\x94gliche R\x81""ckspr\x81nge:",BLACK << 4 | GREEN);
    putIntInBuffer(23,24,&backBuffer[0],lifes,BLACK << 4 | GREEN);
    puttextInBuffer(26,24,&backBuffer[0],8,"Energie:",BLACK << 4 | GREEN);
    putIntInBuffer(35,24,&backBuffer[0],energy,BLACK << 4 | GREEN);
    puttextInBuffer(40,24,&backBuffer[0],7,"Punkte:",BLACK << 4 | GREEN);
    putIntInBuffer(48,24,&backBuffer[0],points,BLACK << 4 | GREEN);
    
    for (int i=shootCount-1;i>=0;i--){
      if ((shoots[i].x-scrollPos >= 80)||(shoots[i].x-scrollPos <0)) {
        shootCount--;
        CopyMemory(&shoots[i],&shoots[i+1],(shootCount-i)*sizeof(shoots[i+1]));
      } else {
        backBuffer[shoots[i].x-scrollPos+shoots[i].y*80+160].Char.AsciiChar='.';
        backBuffer[shoots[i].x-scrollPos+shoots[i].y*80+160].Attributes = LIGHTBLUE << 4 | YELLOW;
      }    
      shoots[i].x += shoots[i].dir;
    }  

    calcDestroyed (levelObjects, levelScreen);

    WriteConsoleOutputA(StdOutput(),&backBuffer[0],rightBottom,topLeft,&screen);

    //COLLISIONTEST SHIP
    if (testShipCollision(levelObjects,shipX,shipY,enableBall,leftFly)) {
      destroyShip(shipX+shipWidth/2-scrollPos,shipY+2,backBuffer,topLeft,rightBottom,screen);
      lifes--;
      if (lifes>0) {
        delete [] levelScreen;
        delete [] levelObjects;
        LoadLevel(globLevelFileName,levelScreen,levelObjects,&shipX,&shipY,&globNextLevelFileName[0],enableBall);
        puttextInBuffer(32,0, &topLines[0],16,&globLevelFileName[0],GREEN);
        energy=100;
        puttextInBuffer(32,0, &topLines[0],16,&globLevelFileName[0],GREEN);
        ball.X=shipX+shipWidth+1;
        ball.Y=shipY+1;
        ballDir.X=2;
        ballDir.Y=0;
        scrollPos=0;
        timeTravel=0;
        if (mouseEnabled) {
          setMousePos(shipX-scrollPos+shipWidth/2,shipY+2+1);
          oldMousePos=grMouse.dwMousePosition;
        }  
      }  else { break;}
    }  
     
    //-----INPUT------
    if (shipX+30>levelWidth) {
      delete [] levelScreen;
      delete [] levelObjects;
      shipX=-1;
      CopyMemory(&globLevelFileName[0],&globNextLevelFileName[0],16);
      LoadLevel(globNextLevelFileName,levelScreen,levelObjects,&shipX,&shipY,&globNextLevelFileName[0],enableBall);
      energy=100;
      if (shipX==-1) break;
      puttextInBuffer(32,0, &topLines[0],16,&globLevelFileName[0],GREEN);
      playShipStart();
      ball.X=shipX+shipWidth+1;
      ball.Y=shipY+1;
      ballDir.X=2;
      ballDir.Y=0;
      scrollPos=0;
      timeTravel=0;
    }  
    if (timeTravel<=0) {
      scrollPos=shipX-1;
      if (scrollPos<0) scrollPos=0;
      else if (scrollPos>=levelWidth-80) scrollPos=levelWidth-80-1;
      else if (!mouseEnabled) shipX++;
      int mouseDir=0;
      if (keypressed(VK_F6)) {
        int size=33;
        char * fileName=new char[size];
        enableMouse(true);
        runInputQuery(41,"Bitte die Zeitmarkierung w\x84hlen (a-z)   " ,GREEN,GREEN,size,&fileName);
        fileName[size]='\x00';
        

        HANDLE file= CreateFile(fileName,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
        DWORD temp;
        ReadFile(file,&globLevelFileName[0],16,&temp,NULL);
        delete [] levelScreen;
        delete [] levelObjects;
        LoadLevel(globLevelFileName,levelScreen,levelObjects,&shipX,&shipY,&globNextLevelFileName[0],enableBall);
        ReadFile(file,&shipX,4,&temp,NULL);
        ReadFile(file,&shipY,4,&temp,NULL);
        ReadFile(file,&scrollPos,4,&temp,NULL);
        ReadFile(file,&ball,8,&temp,NULL);
        ReadFile(file,&ballDir,8,&temp,NULL);
        ReadFile(file,&levelScreen[0],sizeof(levelScreen[0])*(levelWidth*22),&temp,NULL);
        ReadFile(file,&levelObjects[0],sizeof(levelObjects[0])*(levelWidth*22),&temp,NULL);
        ReadFile(file,&lifes,4,&temp,NULL);
        lifes--;
        ReadFile(file,&energy,4,&temp,NULL);
        ReadFile(file,&points,4,&temp,NULL);
        ReadFile(file,&timeTravel,4,&temp,NULL);
        ReadFile(file,&ballKilled,4,&temp,NULL);
        ReadFile(file,&shootCount,4,&temp,NULL);
        for (int i=0;i<shootCount;i++)
          ReadFile(file,&shoots[i],sizeof(shoots[i]),&temp,NULL);
        
        CloseHandle(file);

        const char * ok = "OK";
        if ((file>0) && (GetLastError !=0))
          drawScrollText(39,12,40,12,0,1,&ok,GREEN,GREEN);
        sync(500);
      }  
      if (keypressed(VK_F5)) {
        int size=33;
        char * fileName=new char[size];
        enableMouse(true);
        runInputQuery(41,"Bitte die Zeitmarkierung eingeben (a-z) " ,GREEN,GREEN,size,&fileName);
        fileName[size]='\x00';
        
        HANDLE file= CreateFile(fileName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
        
        DWORD temp;
        WriteFile(file,&globLevelFileName[0],16,&temp,NULL);
        WriteFile(file,&shipX,4,&temp,NULL);
        WriteFile(file,&shipY,4,&temp,NULL);
        WriteFile(file,&scrollPos,4,&temp,NULL);
        WriteFile(file,&ball,8,&temp,NULL);
        WriteFile(file,&ballDir,8,&temp,NULL);
        WriteFile(file,&levelScreen[0],sizeof(levelScreen[0])*(levelWidth*22),&temp,NULL);
        WriteFile(file,&levelObjects[0],sizeof(levelObjects[0])*(levelWidth*22),&temp,NULL);
        WriteFile(file,&lifes,4,&temp,NULL);
        WriteFile(file,&energy,4,&temp,NULL);
        WriteFile(file,&points,4,&temp,NULL);
        WriteFile(file,&timeTravel,4,&temp,NULL);
        WriteFile(file,&ballKilled,4,&temp,NULL);
        WriteFile(file,&shootCount,4,&temp,NULL);
        for (int i=0;i<shootCount;i++)
          WriteFile(file,&shoots[i],sizeof(shoots[i]),&temp,NULL);
        CloseHandle(file);
        const char * ok = "OK";
        if ((file>0) && (GetLastError !=0))
          drawScrollText(39,12,40,12,0,1,&ok,GREEN,GREEN);
        sync(500);
      }  
      if (!(mouseEnabled)) {
        if (keypressed(keyRight)) { shipX++;energy++;if (energy>100) energy = 100;}
        if (keypressed(keyLeft) && (energy>0)) { shipX--; energy--;}
        if (keypressed(keyDown)) { shipY++;}
        if (keypressed(keyUp)) { shipY--;}
        if (keypressed(keyTime)&& (energy >=10)) { 
          timeTravel=30; 
          playShipStart();
          energy-=10;
        }
        if (keypressed(keyShoot) && (ticks-lastShoot >300) && (energy >=5)) { 
          addShoot(shipX+shipWidth,shipY+1,2);
          lastShoot=ticks;
          energy-=5;
          playShoot();
        }
      } else {
       // if (newMouse) {
          if (grMouse.dwEventFlags==0) {
            if (((grMouse.dwButtonState & mouseShoot) == mouseShoot) && (ticks-lastShoot >300) && (energy >=5)) { 
              addShoot(shipX+shipWidth,shipY+1,2);
              lastShoot=ticks;
              energy-=5;
              playShoot();
            }
            if (((grMouse.dwButtonState & mouseTime) == mouseTime) && (energy >=10) && (newMouse)) { 
              timeTravel=30; 
              playShipStart();
              energy-=10;
            }
          } else {
            int oldShipX=shipX;
            int oldShipY=shipY;
/*              if (oldMousePos.X>grMouse.dwMousePosition.X) grMouse.dwMousePosition.X = oldMousePos.X;
              else if (grMouse.dwMousePosition.X>oldMousePos.X+2) grMouse.dwMousePosition.X = oldMousePos.X+2;*/
            shipX=grMouse.dwMousePosition.X-shipWidth/2+scrollPos;
            shipY=grMouse.dwMousePosition.Y-2-1;
            if (scrollPos<levelWidth-81){
              if ((shipX<oldShipX+15)&&(energy>0)) {shipX=oldShipX;energy--;}
              else if (shipX<oldShipX+25) shipX=oldShipX+1;
              else {shipX=oldShipX+2;energy++;if (energy>100) energy = 100;}
            } else {
              if (abs(shipX-oldShipX)>3) 
                if (shipX>oldShipX) shipX=oldShipX+3;
                else shipX=oldShipX-3;
              if (abs(shipY-oldShipY)>3) 
                if (shipY>oldShipY) shipY=oldShipY+3;
                else shipY=oldShipY-3;
            }  
            if (oldMousePos.Y<grMouse.dwMousePosition.Y) mouseDir=1;
            else if (oldMousePos.Y>grMouse.dwMousePosition.Y) mouseDir=-1;
            oldMousePos=grMouse.dwMousePosition;
          }    
          newMouse=false;
        //}  
      }  
      if ((enableBall)&&(!ballKilled)) {
        if (ballDir.X!=0) {
          switch (ballDir.X) {
             case -2: 
/*            if (((ball.X==shipX+shipWidth)||(ball.X==shipX+shipWidth-1))&&(ball.Y>=shipY)&&(ball.Y<=shipY+2)) {
              ballDir.X=abs(ballDir.X);
              break;
            }  
            if (!(moveBallLeft(ball.X,ball.Y,ballDir.X,levelScreen,levelObjects))){ 
              break;
            }  */
            case -1: 
              if (((ball.X==shipX+shipWidth)||(ball.X==shipX+shipWidth-1))&&(ball.Y>=shipY)&&(ball.Y<=shipY+2)) {
                ballDir.X=abs(ballDir.X);
                if ((keypressed(VK_UP)|| (mouseDir<0))) ballDir.Y=-1;
                else if ((keypressed(VK_DOWN)|| (mouseDir>0))) ballDir.Y=1;
                else  ballDir.Y=0;
                break;
              }  
              moveBallLeft(ball.X,ball.Y,ballDir.X,levelScreen,levelObjects);
              if (((ball.X==shipX+shipWidth)||(ball.X==shipX+shipWidth-1))&&(ball.Y>=shipY)&&(ball.Y<=shipY+2)) {
                ballDir.X=abs(ballDir.X);
                if ((keypressed(VK_UP)|| (mouseDir<0))) ballDir.Y=-1;
                else if ((keypressed(VK_DOWN)|| (mouseDir>0))) ballDir.Y=1;
                else  ballDir.Y=0;
                break;
              }  
              break;

            case 2: 
              if (!(moveBallRight(ball.X,ball.Y,ballDir.X,levelScreen,levelObjects)) )
                break;
            case 1: 
              moveBallRight(ball.X,ball.Y,ballDir.X,levelScreen,levelObjects);
              break;
          }  
        
        if (ballDir.Y!=0) {
          switch (ballDir.Y) {
            case -2: 
            /* if (!(moveBallUp(ball.X,ball.Y,ballDir.Y,levelScreen,levelObjects)) )
              break;*/
            case -1: 
              moveBallUp(ball.X,ball.Y,ballDir.Y,levelScreen,levelObjects);
              break;

            case 2: 
/*            if (!(moveBallDown(ball.X,ball.Y,ballDir.Y,levelScreen,levelObjects)) )
              break;*/
            case 1: 
              moveBallDown(ball.X,ball.Y,ballDir.Y,levelScreen,levelObjects);
              break;
          }  
        }  
        if (ballKilled) points -= 2000;
      }  
      }  
      if (shipY<0) shipY = 0;
      if (shipY+shipHeight>=21) {shipY=21-shipHeight;}

    } else {
      timeTravel--;
      if (timeTravel<=0) 
        setMousePos(shipX-scrollPos,shipY+2);
    }
    //COLLISIONTEST
    for (int i=shootCount-1;i>=0;i--){
      if (levelObjects[shoots[i].x+shoots[i].y*levelWidth].type>0) {
        levelObjects[shoots[i].x+shoots[i].y*levelWidth].life--;
        if (levelObjects[shoots[i].x+shoots[i].y*levelWidth].life<0) 
          addDestroyed(shoots[i].x,shoots[i].y,levelObjects,levelScreen);
        shootCount--;
        CopyMemory(&shoots[i],&shoots[i+1],(shootCount-i)*sizeof(shoots[i+1]));
      } 
    }  

    quadroFrame++;
    if ((quadroFrame>4)&&(!timeTravel)) {
      if (energy < 100) energy +=2;
      if (energy > 100) energy = 100;
      quadroFrame=0; 
    }  
    sync(100);
  }
  delete [] topLines;
  delete [] levelScreen;
  delete [] levelObjects;

  puttext (0, 0, 79, 24, cBackupBuffer,aBackupBuffer);
  delete [] cBackupBuffer;
  delete [] aBackupBuffer;
  forceKeyrelease(VK_RETURN);
  sync(100);
}

